@@ -1,7 +1,7 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr 
-#include "common.h"
-RCSID ("$Id: common.cpp,v 1.7 2004/10/27 13:06:46 wojdyr Exp $")
+// $Id: common.cpp,v 1.9 2005/05/28 17:30:18 wojdyr Exp $
 
+#include "common.h"
 #include <string>
 #include <time.h>
 #include "ui.h"
@@ -43,3 +43,10 @@ bool is_double (string s) {
     return (*endptr == 0); 
 }
 
+void replace_all(string &s, const string &old, const string &new_)
+{
+    string::size_type pos = 0; 
+    while ((pos = s.find(old, pos)) != string::npos) 
+        s.replace(pos, old.size(), new_);
+}
+
