@@ -1,5 +1,5 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: wx_dlg.h,v 1.4 2005/02/16 21:04:28 wojdyr Exp $
+// $Id: wx_dlg.h,v 1.12 2005/06/04 13:23:52 wojdyr Exp $
 
 #ifndef WX_DLG__H__
 #define WX_DLG__H__
@@ -8,6 +8,7 @@
 #include <wx/notebook.h>
 #include <wx/spinctrl.h>
 #include <wx/listctrl.h> 
+#include <wx/dirctrl.h>
 #include "pag.h"
 
 class FuncTree;
@@ -102,53 +103,36 @@ protected:
     DECLARE_EVENT_TABLE()
 };
 
-class FDStdDevDlg : public wxDialog
-{
-public:
-    std::string get_command();
-    FDStdDevDlg (wxWindow* parent, wxWindowID id);
-protected:
-    wxTextCtrl *tc_val;
-    wxString val;
-    wxRadioButton *rb0, *rb1;
-    DECLARE_EVENT_TABLE()
-};
 
-class FDRangeDlg : public wxDialog
+class FDXLoadDlg;
+
+/// helper class used in FDXLoadDlg
+class LoadDataDirCtrl : public wxGenericDirCtrl
 {
 public:
-    wxTextCtrl *tc_from, *tc_to;
-    wxStaticText *label_from, *label_to;
-    wxString from, to;
-    wxRadioBox *rb;
-    FDRangeDlg (wxWindow* parent, wxWindowID id);
-protected:
-    void OnRadioBoxSelection (wxCommandEvent& event);
-    void OnApplyButton (wxCommandEvent& event);
+    LoadDataDirCtrl(FDXLoadDlg* parent);
+    void OnPathSelectionChanged(wxTreeEvent &event);
+    void SetFilterIndex(int n);
     DECLARE_EVENT_TABLE()
+private:
+    FDXLoadDlg *load_dlg;
 };
 
 class FDXLoadDlg : public wxDialog
 {
 public:
-    std::string filename;
     FDXLoadDlg (wxWindow* parent, wxWindowID id);
     std::string get_command();
-    void OnChangeButton (wxCommandEvent& event);
-    void set_filename (const std::string &path);
+    std::string get_filename();
+    void on_filter_change();
+    void on_path_change();
 
 protected:
-    wxTextCtrl *file_txt_ctrl;
-    wxSpinCtrl *x_column, *y_column, *s_column, 
-               *from_range, *to_range, *from_every, 
-               *to_every, *of_every, *merge_number; 
-    wxPanel *columns_panel, *other_types_panel;
-    wxRadioBox *rb_filetype, *yrbox;
-    wxCheckBox *merge_cb, *std_dev_cb;
-    wxListBox *lb_filetypes;
-    void OnFTypeRadioBoxSelection (wxCommandEvent& event);
-    void OnSelRadioBoxSelection (wxCommandEvent& event);
-    void OnMergeCheckBox (wxCommandEvent& event);
+    LoadDataDirCtrl *dir_ctrl;
+    wxTextCtrl *filename_tc;
+    wxSpinCtrl *x_column, *y_column, *s_column;
+    wxPanel *columns_panel;
+    wxCheckBox *std_dev_cb, *append_cb;
     void OnStdDevCheckBox (wxCommandEvent& event);
     DECLARE_EVENT_TABLE()
 };
@@ -177,18 +161,86 @@ protected:
 };
 
 
+struct DataTransExample
+{
+    std::string name;
+    std::string category;
+    std::string description;
+    std::string code;
+    bool in_menu;
+
+    DataTransExample(const std::string& name_, const std::string& category_, 
+                     const std::string& description_, const std::string& code_,
+                     bool in_menu_=false)
+        : name(name_), category(category_), description(description_),
+          code(code_), in_menu(in_menu_) {}
+   DataTransExample(std::string line);
+   std::string as_fileline() const;
+};
+
 class DataEditorDlg : public wxDialog
 {
 public:
     DataEditorDlg (wxWindow* parent, wxWindowID id, Data *data_);
     void OnRevert (wxCommandEvent& event);
+    void OnSaveAs (wxCommandEvent& event);
+    void OnAdd (wxCommandEvent& event);
+    void OnRemove (wxCommandEvent& event);
+    void OnUp (wxCommandEvent& event);
+    void OnDown (wxCommandEvent& event);
+    void OnSave (wxCommandEvent& event);
+    void OnReset (wxCommandEvent& event);
+    void OnApply (wxCommandEvent& event);
+    void OnHelp (wxCommandEvent& event);
+    void OnClose (wxCommandEvent& event);
+    void OnCodeText (wxCommandEvent&) { CodeText(); }
+    void CodeText();
+    void OnESelected (wxListEvent&) { ESelected(); }
+    void ESelected();
+    void OnEActivated (wxListEvent& event);
     void update_data(Data *data_);
+    static const std::vector<DataTransExample>& get_examples() 
+                                                    { return examples; }
+    static void read_examples(bool reset=false);
+    static void execute_tranform(std::string code);
 protected:
+    static std::vector<DataTransExample> examples;
     wxGrid *grid;
     Data *data;
-    wxStaticText *filename_label, *title_label;
+    wxStaticText *filename_label, *title_label, *description;
+    wxListCtrl *example_list; 
+    wxTextCtrl *code;
+    wxButton *revert_btn, *save_as_btn, *apply_btn, *help_btn,
+             *add_btn, *remove_btn, *up_btn, *down_btn, 
+             *save_btn, *reset_btn;
+
+    void initialize_examples(bool reset=false);
+    int get_selected_item();
+    void insert_example_list_item(int n);
+    void select_example(int item);
     DECLARE_EVENT_TABLE()
 };
 
+
+class ExampleEditorDlg : public wxDialog
+{
+public:
+    ExampleEditorDlg(wxWindow* parent, wxWindowID id, DataTransExample& ex_,
+                     const std::vector<DataTransExample>& examples_, int pos_);
+    void OnOK(wxCommandEvent &event);
+protected:
+    DataTransExample& ex;
+    const std::vector<DataTransExample>& examples;
+    int pos;
+    wxTextCtrl *name_tc, *description_tc, *code_tc;
+    wxComboBox *category_c;
+    wxCheckBox *inmenu_cb;
+
+    DECLARE_EVENT_TABLE()
+};
+
+wxString get_user_conffile(const wxString &filename);
+bool export_data_dlg(wxWindow *parent, bool load_exported=false);
+
 #endif
 
