@@ -1,4 +1,5 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
+// $Id: wx_gui.cpp,v 1.44 2005/06/08 20:52:28 wojdyr Exp $
 
 // wxwindows headers, see wxwindows samples for description
 #include <wx/wxprec.h>
@@ -9,8 +10,6 @@
 #include <wx/wx.h>
 #endif
 
-#include "common.h"
-RCSID ("$Id: wx_gui.cpp,v 1.32 2005/03/04 15:13:03 wojdyr Exp $")
 
 #include <wx/laywin.h>
 #include <wx/splitter.h>
@@ -32,8 +31,11 @@ RCSID ("$Id: wx_gui.cpp,v 1.32 2005/03/04 15:13:03 wojdyr Exp $")
 #include <algorithm>
 #include <locale.h>
 #include <string.h>
+#include <boost/spirit/version.hpp> //SPIRIT_VERSION
 
+#include "common.h"
 #include "wx_plot.h"
+#include "wx_mplot.h"
 #include "wx_gui.h"
 #include "wx_dlg.h"
 #include "wx_pane.h"
@@ -55,7 +57,6 @@ RCSID ("$Id: wx_gui.cpp,v 1.32 2005/03/04 15:13:03 wojdyr Exp $")
 #endif
 
 //toolbars icons
-#include "img/clear.xpm"     
 #include "img/plusbg.xpm"  
 //#include "img/spline.xpm"
 #include "img/autoadd.xpm"     
@@ -89,8 +90,9 @@ char* about_html =
 "  <table cellspacing=3 cellpadding=4 width=100%>                        "
 "   <tr bgcolor=#101010 align=center> <td>                               "
 "      <h1> fityk " VERSION "</h1>                                       "
-"      <h6> powered by " wxVERSION_STRING "</h6>                         "
-"      <p><font size=-2>$Date: 2005/03/04 15:13:03 $</font></p>          "
+"      <h6> powered by " wxVERSION_STRING "<br>                          "
+"           powered by Boost.Spirit %d.%d.%d</h6>                        "
+"      <p><font size=-2>$Date: 2005/06/08 20:52:28 $</font></p>          "
 "   </td> </tr>                                                          "
 "   <tr> <td bgcolor=#73A183>                                            "
 "     <font color=black>                                                 "
@@ -120,17 +122,13 @@ enum {
     ID_H_TIP                   ,
     ID_D_LOAD                  ,
     ID_D_XLOAD                 ,
-    ID_D_RECENT        = 24010 , //and next ones
-    ID_D_INFO          = 24130 ,
+    ID_D_RECENT                , //and next ones
+    ID_D_RECENT_END = ID_D_RECENT+30 , 
+    ID_D_INFO                  ,
     ID_D_EDITOR                ,
-    ID_D_DEVIATION             ,
-    ID_D_RANGE                 ,
-    ID_D_B_INFO                ,
-    ID_D_B_CLEAR               ,
-    ID_D_BACKGROUND            ,
-    ID_D_CALIBRATE             ,
+    ID_D_FDT                   ,
+    ID_D_FDT_END = ID_D_FDT+50 ,
     ID_D_EXPORT                ,
-    ID_D_SET                   ,
     ID_S_ADD                   ,
     ID_S_HISTORY               ,
     ID_S_INFO                  ,
@@ -146,8 +144,9 @@ enum {
     ID_F_CONTINUE              ,
     ID_F_INFO                  ,
     ID_F_SET                   ,
-    ID_F_M             = 24160 , // and a few next IDs
-    ID_C_WAVELENGTH    = 24171 ,
+    ID_F_M                     , 
+    ID_F_M_END = ID_F_M+10     , 
+    ID_C_WAVELENGTH            ,
     ID_C_ADD                   ,
     ID_C_INFO                  ,
     ID_C_REMOVE                ,
@@ -168,9 +167,14 @@ enum {
     ID_G_M_RANGE               ,
     ID_G_M_BG                  ,
     ID_G_M_ADD                 ,
+    ID_G_M_BG_STRIP            ,
+    ID_G_M_BG_CLEAR            ,
+    ID_G_M_BG_SPLINE           ,
+    ID_G_M_BG_SUB              ,
     ID_G_M_PEAK                ,
-    ID_G_M_PEAK_N      = 24220 ,
-    ID_G_SHOW          = 24260 ,
+    ID_G_M_PEAK_N              ,
+    ID_G_M_PEAK_N_END = ID_G_M_PEAK_N+40 ,
+    ID_G_SHOW                  ,
     ID_G_S_TOOLBAR             ,
     ID_G_S_STATBAR             ,
     ID_G_S_DPANE               ,
@@ -182,8 +186,9 @@ enum {
     ID_G_V_VERT                ,
     ID_G_V_SCROLL_L            ,
     ID_G_V_SCROLL_R            ,
-    ID_G_V_ZOOM_PREV   = 24302 ,
-    ID_G_LCONF1        = 24342 ,
+    ID_G_V_ZOOM_PREV           ,
+    ID_G_V_ZOOM_PREV_END = ID_G_V_ZOOM_PREV+40 ,
+    ID_G_LCONF1                ,
     ID_G_LCONF2                ,
     ID_G_LCONFB                ,
     ID_G_SCONF                 ,
@@ -195,7 +200,7 @@ enum {
     ID_ft_m_bg                 ,
     ID_ft_m_add                ,
     ID_ft_v_pr                 ,
-    ID_ft_b_with               ,
+    ID_ft_b_strip              ,
     ID_ft_f_run                ,
     ID_ft_f_cont               ,
     ID_ft_f_undo               ,
@@ -258,6 +263,8 @@ bool FApp::OnInit(void)
                                         wxCONFIG_USE_LOCAL_FILE);
     wxConfig::Set(config);
 
+    DataEditorDlg::read_examples();
+
     // Create the main frame window
     frame = new FFrame(NULL, -1, app_name, wxDEFAULT_FRAME_STYLE);
 
@@ -276,12 +283,8 @@ bool FApp::OnInit(void)
 
     SetTopWindow(frame);
 
-    //run initial commands (from ~/.fityk/init file)
-    wxString fityk_dir = wxGetHomeDir() + wxFILE_SEP_PATH + config_dirname;
-    if (!wxDirExists(fityk_dir))
-        wxMkdir(fityk_dir);
-    wxString startup_file_path = fityk_dir + wxFILE_SEP_PATH 
-                                + startup_commands_filename;
+    // run initial commands (from ~/.fityk/init file)
+    wxString startup_file_path = get_user_conffile(startup_commands_filename);
     if (wxFileExists(startup_file_path)) {
         getUI()->execScript(startup_file_path.c_str());
     }
@@ -367,16 +370,12 @@ BEGIN_EVENT_TABLE(FFrame, wxFrame)
 
     EVT_MENU (ID_D_LOAD,        FFrame::OnDLoad)   
     EVT_MENU (ID_D_XLOAD,       FFrame::OnDXLoad)   
-    EVT_MENU_RANGE (ID_D_RECENT+1, ID_D_RECENT+100, FFrame::OnDRecent)
+    EVT_MENU_RANGE (ID_D_RECENT+1, ID_D_RECENT_END, FFrame::OnDRecent)
     EVT_MENU (ID_D_EDITOR,      FFrame::OnDEditor)
+    EVT_UPDATE_UI (ID_D_FDT,    FFrame::OnFastDTUpdate)
+    EVT_MENU_RANGE (ID_D_FDT+1, ID_D_FDT_END, FFrame::OnFastDT)
     EVT_MENU (ID_D_INFO,        FFrame::OnDInfo)
-    EVT_MENU (ID_D_DEVIATION  , FFrame::OnDDeviation)  
-    EVT_MENU (ID_D_RANGE,       FFrame::OnDRange)
-    EVT_MENU (ID_D_B_INFO,      FFrame::OnDBInfo)
-    EVT_MENU (ID_D_B_CLEAR,     FFrame::OnDBClear)
-    EVT_MENU (ID_D_CALIBRATE,   FFrame::OnDCalibrate)  
     EVT_MENU (ID_D_EXPORT,      FFrame::OnDExport) 
-    EVT_MENU (ID_D_SET,         FFrame::OnDSet) 
 
     EVT_MENU (ID_S_HISTORY,     FFrame::OnSHistory)    
     EVT_MENU (ID_S_INFO,        FFrame::OnSInfo)    
@@ -391,7 +390,7 @@ BEGIN_EVENT_TABLE(FFrame, wxFrame)
     EVT_MENU (ID_M_SET,         FFrame::OnMSet)
 
     EVT_UPDATE_UI (ID_F_METHOD, FFrame::OnFMethodUpdate)
-    EVT_MENU_RANGE (ID_F_M+0, ID_F_M+8, FFrame::OnFOneOfMethods)    
+    EVT_MENU_RANGE (ID_F_M+0, ID_F_M_END, FFrame::OnFOneOfMethods)    
     EVT_MENU (ID_F_RUN,         FFrame::OnFRun)    
     EVT_MENU (ID_F_CONTINUE,    FFrame::OnFContinue)    
     EVT_MENU (ID_F_INFO,        FFrame::OnFInfo)    
@@ -422,7 +421,10 @@ BEGIN_EVENT_TABLE(FFrame, wxFrame)
     EVT_MENU (ID_G_M_BG,        FFrame::OnChangeMouseMode)
     EVT_MENU (ID_G_M_ADD,       FFrame::OnChangeMouseMode)
     EVT_UPDATE_UI (ID_G_M_PEAK, FFrame::OnModePeak)
-    EVT_MENU_RANGE (ID_G_M_PEAK_N+0, ID_G_M_PEAK_N+30, FFrame::OnChangePeakType)
+    EVT_MENU_RANGE (ID_G_M_PEAK_N, ID_G_M_PEAK_N_END, FFrame::OnChangePeakType)
+    EVT_MENU (ID_G_M_BG_STRIP,  FFrame::OnStripBg)
+    EVT_MENU (ID_G_M_BG_CLEAR,  FFrame::OnClearBg)
+    EVT_MENU (ID_G_M_BG_SPLINE, FFrame::OnSplineBg)
     EVT_UPDATE_UI (ID_G_SHOW,   FFrame::OnGuiShowUpdate)
     EVT_MENU (ID_G_S_DPANE,     FFrame::OnSwitchDPane)
     EVT_MENU_RANGE (ID_G_S_A1, ID_G_S_A2, FFrame::OnSwitchAuxPlot)
@@ -435,7 +437,7 @@ BEGIN_EVENT_TABLE(FFrame, wxFrame)
     EVT_MENU (ID_G_V_SCROLL_L,  FFrame::OnGScrollLeft)
     EVT_MENU (ID_G_V_SCROLL_R,  FFrame::OnGScrollRight)
     EVT_UPDATE_UI (ID_G_V_ZOOM_PREV, FFrame::OnShowMenuZoomPrev)
-    EVT_MENU_RANGE (ID_G_V_ZOOM_PREV+1, ID_G_V_ZOOM_PREV+20, 
+    EVT_MENU_RANGE (ID_G_V_ZOOM_PREV+1, ID_G_V_ZOOM_PREV_END, 
                                 FFrame::OnPreviousZoom)    
     EVT_MENU (ID_G_LCONF1,      FFrame::OnConfigRead)
     EVT_MENU (ID_G_LCONF2,      FFrame::OnConfigRead)
@@ -458,7 +460,7 @@ FFrame::FFrame(wxWindow *parent, const wxWindowID id, const wxString& title,
                  const long style)
     : wxFrame(parent, id, title, wxDefaultPosition, wxDefaultSize, style), 
       main_pane(0), data_pane(0), status_bar(0), 
-      peak_type_nr(0), toolbar(0), dxload_dialog(0), data_editor(0),
+      peak_type_nr(0), toolbar(0), 
       print_data(new wxPrintData), page_setup_data(new wxPageSetupData),
 #ifdef __WXMSW__
       help()
@@ -492,9 +494,7 @@ FFrame::FFrame(wxWindow *parent, const wxWindowID id, const wxString& title,
     status_bar = new FStatusBar(this);
     SetStatusBar(status_bar);
 
-    SetAutoLayout (true);
     SetSizer(sizer);
-    sizer->Fit(this);
     sizer->SetSizeHints(this);
 
 
@@ -586,21 +586,21 @@ void FFrame::read_settings(wxConfigBase *cf)
 {
     // restore window layout, frame position and size
     cf->SetPath("/Frame");
-    SwitchToolbar(from_config_read_bool(cf, "ShowToolbar", true));
-    SwitchStatbar(from_config_read_bool(cf, "ShowStatbar", true));
+    SwitchToolbar(read_bool_from_config(cf, "ShowToolbar", true));
+    SwitchStatbar(read_bool_from_config(cf, "ShowStatbar", true));
     int x = cf->Read("x", 50),
         y = cf->Read("y", 50),
         w = cf->Read("w", 650),
         h = cf->Read("h", 400);
     Move(x, y);
     SetClientSize(w, h);
-    v_splitter->SetProportion(from_config_read_double(cf, "VertSplitProportion",
+    v_splitter->SetProportion(read_double_from_config(cf, "VertSplitProportion",
                                                           0.8));
-    SwitchDPane(from_config_read_bool(cf, "ShowDataPane", false));
-    main_pane->SetProportion(from_config_read_double(cf, "MainPaneProportion",
+    SwitchDPane(read_bool_from_config(cf, "ShowDataPane", false));
+    main_pane->SetProportion(read_double_from_config(cf, "MainPaneProportion",
                                                           0.7));
-    SwitchIOPane(from_config_read_bool(cf, "ShowIOPane", true));
-    SwitchCrosshair(from_config_read_bool(cf, "ShowCrosshair", false));
+    SwitchIOPane(read_bool_from_config(cf, "ShowIOPane", true));
+    SwitchCrosshair(read_bool_from_config(cf, "ShowCrosshair", false));
 }
 
 void FFrame::save_all_settings(wxConfigBase *cf) const
@@ -648,19 +648,12 @@ void FFrame::set_menubar()
     data_menu->Append(ID_D_RECENT, "Recent &Files", data_menu_recent); 
     data_menu->AppendSeparator();
 
-    data_menu->Append (ID_D_EDITOR,   "&Viewer", "Open data viewer");
+    data_menu->Append (ID_D_EDITOR,   "&Editor", "Open data editor");
+    this->data_ft_menu = new wxMenu;
+    data_menu->Append (ID_D_FDT,      "&Fast Transformations", data_ft_menu, 
+                                      "Quick data transformations");
     data_menu->Append (ID_D_INFO,     "&Info", "Info about loaded data");
-    data_menu->Append (ID_D_DEVIATION  , "Std. &Dev.", 
-                                            "Change errors for data points");
-    data_menu->Append (ID_D_RANGE,    "&Range", "Change active points");
-    wxMenu* data_bg_menu = new wxMenu;
-    data_bg_menu->Append(ID_D_B_INFO, "&Info", "Info about baseline points");
-    data_bg_menu->Append(ID_D_B_CLEAR, "&Clear", "Clear all baseline points");
-    data_menu->Append (ID_D_BACKGROUND, "&Background", data_bg_menu);
-    data_menu->Append (ID_D_CALIBRATE, "&Calibrate",
-                       "Define calibration curve (command line only)");
     data_menu->Append (ID_D_EXPORT,   "&Export", "Save data to file");
-    data_menu->Append (ID_D_SET,      "&Settings", "Preferences and options");
 
     wxMenu* sum_menu = new wxMenu;
     sum_menu->Append (ID_S_HISTORY,   "&History", "Go back or forward"
@@ -718,7 +711,7 @@ void FFrame::set_menubar()
     gui_menu_mode->AppendRadioItem (ID_G_M_RANGE, "&Range", 
                              "Use mouse for activating and disactivating data");
     gui_menu_mode->AppendRadioItem (ID_G_M_BG, "&Baseline", 
-                                    "Use mouse for substracting background");
+                                    "Use mouse for subtracting background");
     gui_menu_mode->AppendRadioItem (ID_G_M_ADD, "&Peak-Add", 
                                     "Use mouse for adding new peaks");
     wxMenu* gui_menu_mode_peak = new wxMenu;
@@ -729,6 +722,17 @@ void FFrame::set_menubar()
                                             (*i)->name.c_str());
     gui_menu_mode->AppendSeparator();
     gui_menu_mode->Append (ID_G_M_PEAK, "Peak &type", gui_menu_mode_peak);
+    gui_menu_mode->AppendSeparator();
+    wxMenu* baseline_menu = new wxMenu;
+    baseline_menu->Append (ID_G_M_BG_STRIP, "&Strip baseline", 
+                           "Subtract selected baseline from data");
+    baseline_menu->Append (ID_G_M_BG_CLEAR, "&Clear baseline", 
+                           "Clear baseline points");
+    baseline_menu->AppendCheckItem (ID_G_M_BG_SPLINE, "&Spline interpolation", 
+                                    "Cubic spline interpolation of points");
+    baseline_menu->Check(ID_G_M_BG_SPLINE, true);
+    gui_menu_mode->Append(ID_G_M_BG_SUB, "Baseline handling", baseline_menu);
+    gui_menu_mode->Enable(ID_G_M_BG_SUB, false);
     gui_menu->Append(ID_G_MODE, "&Mode", gui_menu_mode);
     gui_menu->AppendSeparator();
     wxMenu* gui_menu_show = new wxMenu;
@@ -855,23 +859,29 @@ void FFrame::OnShowHelp(wxCommandEvent& WXUNUSED(event))
         help.DisplayContents();
 }
 
+bool FFrame::display_help_section(const string &s)
+{
+    return help.DisplaySection(s.c_str());
+}
+
 void FFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
 {
     wxDialog* dlg = new wxDialog(this, -1, wxString("About..."));
     wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
     wxHtmlWindow *html = new wxHtmlWindow (dlg, -1, 
-                                           wxDefaultPosition, wxSize(300, 160), 
+                                           wxDefaultPosition, wxSize(400, 160), 
                                            wxHW_SCROLLBAR_NEVER);
     html->SetBorders(0);
-    html->SetPage(about_html);
+    html->SetPage(wxString::Format(about_html, SPIRIT_VERSION / 0x1000,
+                                               SPIRIT_VERSION % 0x1000 / 0x0100,
+                                               SPIRIT_VERSION % 0x0100));
     html->SetSize (html->GetInternalRepresentation()->GetWidth(), 
                    html->GetInternalRepresentation()->GetHeight());
     topsizer->Add (html, 1, wxALL, 10);
-    topsizer->Add (new wxStaticLine(dlg, -1), 0, wxEXPAND|wxLEFT|wxRIGHT, 10);
+    //topsizer->Add (new wxStaticLine(dlg, -1), 0, wxEXPAND|wxLEFT|wxRIGHT, 10);
     wxButton *bu_ok = new wxButton (dlg, wxID_OK, "OK");
     bu_ok->SetDefault();
-    topsizer->Add (bu_ok, 0, wxALL | wxALIGN_RIGHT, 15);
-    dlg->SetAutoLayout(true);
+    topsizer->Add (bu_ok, 0, wxALL|wxEXPAND, 10);
     dlg->SetSizer(topsizer);
     topsizer->Fit(dlg);
     dlg->ShowModal();
@@ -918,21 +928,10 @@ void FFrame::OnDLoad (wxCommandEvent& WXUNUSED(event))
 
 void FFrame::OnDXLoad (wxCommandEvent& WXUNUSED(event))
 {
-    if (!dxload_dialog)
-        dxload_dialog = new FDXLoadDlg(this, -1);
-
-    string fname = my_data->get_filename();
-    if (fname.empty()) {
-        dxload_dialog->OnChangeButton(dummy_cmd_event);
-        if (dxload_dialog->filename.empty())
-            return;
-    }
-    else
-        dxload_dialog->set_filename(fname);
-
-    if (dxload_dialog->ShowModal() == wxID_OK) {
-        exec_command (dxload_dialog->get_command());
-        add_recent_data_file(dxload_dialog->filename.c_str());
+    FDXLoadDlg dxload_dialog(this, -1);
+    if (dxload_dialog.ShowModal() == wxID_OK) {
+        exec_command (dxload_dialog.get_command());
+        add_recent_data_file(dxload_dialog.get_filename().c_str());
     }
 }
 
@@ -945,13 +944,44 @@ void FFrame::OnDRecent (wxCommandEvent& event)
 
 void FFrame::OnDEditor (wxCommandEvent& WXUNUSED(event))
 {
-    if (!data_editor) {
-        data_editor = new DataEditorDlg(this, -1, my_data);
-        data_editor->Show();
-    }
-    else {
-        data_editor->update_data(my_data);
+    DataEditorDlg data_editor(this, -1, my_data);
+    data_editor.ShowModal();
+}
+
+void FFrame::OnFastDTUpdate (wxUpdateUIEvent& event)
+{
+    const vector<DataTransExample> &all = DataEditorDlg::get_examples();
+    vector<DataTransExample> examples;
+    for (vector<DataTransExample>::const_iterator i = all.begin(); 
+            i != all.end(); ++i)
+        if (i->in_menu)
+            examples.push_back(*i);
+    int menu_len = data_ft_menu->GetMenuItemCount();
+    for (int i = 0; i < size(examples); ++i) {
+        int id = ID_D_FDT+i+1;
+        wxString name = examples[i].name.c_str();
+        if (i >= menu_len)
+            data_ft_menu->Append(id, name);
+        else if (data_ft_menu->GetLabel(id) == name)
+            continue;
+        else
+            data_ft_menu->SetLabel(id, name);
     }
+    for (int i = size(examples); i < menu_len; ++i) 
+        data_ft_menu->Delete(ID_D_FDT+i+1);
+    event.Skip();
+}
+
+void FFrame::OnFastDT (wxCommandEvent& event)
+{
+    string name = GetMenuBar()->GetLabel(event.GetId()).c_str();
+    const vector<DataTransExample> &examples = DataEditorDlg::get_examples();
+    for (vector<DataTransExample>::const_iterator i = examples.begin();
+            i != examples.end(); ++i)
+        if (i->name == name) {
+            DataEditorDlg::execute_tranform(i->code);
+            return;
+        }
 }
 
 void FFrame::OnDInfo (wxCommandEvent& WXUNUSED(event))
@@ -959,59 +989,9 @@ void FFrame::OnDInfo (wxCommandEvent& WXUNUSED(event))
     exec_command ("d.info");
 }
 
-void FFrame::OnDDeviation  (wxCommandEvent& WXUNUSED(event))
-{
-    FDStdDevDlg *dialog = new FDStdDevDlg(this, -1);
-    if (dialog->ShowModal() == wxID_OK) {
-        exec_command (dialog->get_command());
-    }
-    dialog->Destroy();
-}
-
-void FFrame::OnDRange (wxCommandEvent& WXUNUSED(event))
-{
-    FDRangeDlg *dialog = new FDRangeDlg(this, -1);
-    dialog->ShowModal();
-    dialog->Destroy();
-}
-
-void FFrame::OnDBInfo (wxCommandEvent& WXUNUSED(event))
-{
-    exec_command(S("d.background")); 
-}
-
-void FFrame::OnDBClear (wxCommandEvent& WXUNUSED(event))
-{
-    exec_command(S("d.background !")); 
-}
-
-void FFrame::OnDCalibrate (wxCommandEvent& WXUNUSED(event))
-{
-    wxMessageBox (("This menu item is only reminding about command d.calibarate"
-                   ".\n If you don't know how to use this command,\n"
-                   " look for it in the manual.\n"
-                   "By the way, is anyone using command d.calibrate?"),
-                  "d.calibrate", wxOK|wxICON_INFORMATION);
-}
-
 void FFrame::OnDExport (wxCommandEvent& WXUNUSED(event))
 {
-    static int filter_idx = 0;
-    static wxString dir = ".";
-    wxFileDialog fdlg (frame, "Export data to file", dir, "",
-                       "x y data (*.dat)|*.dat;*.DAT"
-                       "|fityk file (*.fit)|*.fit;*.FIT",
-                       wxSAVE | wxOVERWRITE_PROMPT);
-    fdlg.SetFilterIndex(filter_idx);
-    if (fdlg.ShowModal() == wxID_OK) 
-        exec_command (("d.export '" + fdlg.GetPath() + "'").c_str());
-    filter_idx = fdlg.GetFilterIndex();
-    dir = fdlg.GetDirectory();
-}
-
-void FFrame::OnDSet (wxCommandEvent& WXUNUSED(event))
-{
-    OnXSet ("Data", 'd');
+    export_data_dlg(this);
 }
 
 void FFrame::OnSHistory      (wxCommandEvent& WXUNUSED(event))
@@ -1066,7 +1046,7 @@ void FFrame::OnSExport       (wxCommandEvent& WXUNUSED(event))
     static int filter_idx = 0;
     static wxString dir = ".";
     wxString name = wxFileName(my_data->get_filename().c_str()).GetName();
-    wxFileDialog fdlg (frame, "Export curve to file", dir, name,
+    wxFileDialog fdlg (this, "Export curve to file", dir, name,
                        "parameters of peaks (*.peaks)|*.peaks"
                        "|x y data (*.dat)|*.dat;*.DAT"
                        "|XFIT peak listing (*xfit.txt)|*xfit.txt;*XFIT.TXT" 
@@ -1171,7 +1151,7 @@ void FFrame::OnCSet          (wxCommandEvent& WXUNUSED(event))
 
 void FFrame::OnOLog          (wxCommandEvent& WXUNUSED(event))
 {
-    wxFileDialog fdlg (frame, "Log to file", "", "",
+    wxFileDialog fdlg (this, "Log to file", "", "",
                        "Log input to file (*.fit)|*.fit;*.FIT"
                        "|Log output to file (*.fit)|*.fit;*.FIT"
                        "|Log input & output (*.fit)|*.fit;*.FIT"
@@ -1215,7 +1195,7 @@ void FFrame::OnO_Reset   (wxCommandEvent& WXUNUSED(event))
 void FFrame::OnOInclude      (wxCommandEvent& WXUNUSED(event))
 {
     static wxString dir = ".";
-    wxFileDialog fdlg (frame, "Execute commands from file", dir, "",
+    wxFileDialog fdlg (this, "Execute commands from file", dir, "",
                               "fityk file (*.fit)|*.fit;*.FIT|all files|*",
                               wxOPEN | wxFILE_MUST_EXIST);
     if (fdlg.ShowModal() == wxID_OK) {
@@ -1241,7 +1221,7 @@ void FFrame::OnOWait         (wxCommandEvent& WXUNUSED(event))
          
 void FFrame::OnODump         (wxCommandEvent& WXUNUSED(event))
 {
-    wxFileDialog fdlg (frame, "Dump current program state to file as script", 
+    wxFileDialog fdlg (this, "Dump current program state to file as script", 
                                 "", "", "fityk file (*.fit)|*.fit;*.FIT",
                        wxSAVE | wxOVERWRITE_PROMPT);
     if (fdlg.ShowModal() == wxID_OK) 
@@ -1296,38 +1276,60 @@ void FFrame::OnXSet (string name, char letter)
 
 void FFrame::OnChangeMouseMode (wxCommandEvent& event)
 {
+    const MainPlot* plot = plot_pane->get_plot();
+    if (plot->get_mouse_mode() == mmd_bg && !plot->bg_empty()) {
+        int r = wxMessageBox("You have selected the baseline, but have not\n"
+                             "stripped it. Do you want to change data\n"
+                              "and strip selected baseline?\n" 
+                              "If you want to stay in background mode,\n"
+                              "press Cancel.",
+                              "Want to strip background?", 
+                              wxICON_QUESTION|wxYES_NO|wxCANCEL);
+        if (r == wxYES)
+            plot_pane->get_bg_manager()->strip_background();
+        else if (r == wxNO)
+            plot_pane->get_bg_manager()->clear_background();
+        else { //wxCANCEL
+            GetMenuBar()->Check(ID_G_M_BG, true);
+            if (toolbar) 
+                toolbar->ToggleTool(ID_ft_m_bg, true);
+            return;
+        }
+    }
     Mouse_mode_enum mode = mmd_zoom;
     switch (event.GetId()) {
         case ID_G_M_ZOOM:   
         case ID_ft_m_zoom:   
             mode = mmd_zoom;   
             GetMenuBar()->Check(ID_G_M_ZOOM, true);
-            if (toolbar) toolbar->ToggleTool(ID_ft_m_zoom, true);
+            if (toolbar) 
+                toolbar->ToggleTool(ID_ft_m_zoom, true);
             break;
         case ID_G_M_RANGE:   
         case ID_ft_m_range:  
             mode = mmd_range;  
             GetMenuBar()->Check(ID_G_M_RANGE, true);
-            if (toolbar) toolbar->ToggleTool(ID_ft_m_range, true);
+            if (toolbar) 
+                toolbar->ToggleTool(ID_ft_m_range, true);
             break;
         case ID_G_M_BG:   
         case ID_ft_m_bg:     
             mode = mmd_bg;     
             GetMenuBar()->Check(ID_G_M_BG, true);
-            if (toolbar) toolbar->ToggleTool(ID_ft_m_bg, true);
+            if (toolbar) 
+                toolbar->ToggleTool(ID_ft_m_bg, true);
             break;
         case ID_G_M_ADD:   
         case ID_ft_m_add:    
             mode = mmd_add;    
             GetMenuBar()->Check(ID_G_M_ADD, true);
-            if (toolbar) toolbar->ToggleTool(ID_ft_m_add, true);
+            if (toolbar) 
+                toolbar->ToggleTool(ID_ft_m_add, true);
             break;  
         default: assert(0);
     }
-    if (mode==mmd_bg && toolbar && toolbar->GetToolState(ID_ft_b_with)==false) {
-        toolbar->ToggleTool(ID_ft_b_with, true);
-        exec_command ("o.plot +");
-    }
+    toolbar->EnableTool(ID_ft_b_strip, (mode == mmd_bg));
+    GetMenuBar()->Enable(ID_G_M_BG_SUB, (mode == mmd_bg));
     plot_pane->set_mouse_mode(mode);
 }
 
@@ -1343,6 +1345,23 @@ void FFrame::OnChangePeakType(wxCommandEvent& event)
     if (toolbar) toolbar->update_peak_type();
 }
 
+void FFrame::OnStripBg(wxCommandEvent& WXUNUSED(event))
+{
+    plot_pane->get_bg_manager()->strip_background();
+}
+
+void FFrame::OnClearBg(wxCommandEvent& WXUNUSED(event))
+{
+    plot_pane->get_bg_manager()->clear_background();
+    refresh_plots(true, false, true);
+}
+
+void FFrame::OnSplineBg(wxCommandEvent& event)
+{
+    plot_pane->get_bg_manager()->set_spline_bg(event.IsChecked());
+    refresh_plots(true, false, true);
+}
+
 void FFrame::SwitchToolbar(bool show)
 {
     if (show && !GetToolBar()) {
@@ -1488,6 +1507,7 @@ void FFrame::OnConfigRead (wxCommandEvent& event)
 
 void FFrame::OnConfigBuiltin (wxCommandEvent& WXUNUSED(event))
 {
+    // fake config file
     wxString name = wxString(config_dirname) + wxFILE_SEP_PATH+"builtin-config";
     wxConfigBase *config = new wxConfig("", "", name, "", 
                                         wxCONFIG_USE_LOCAL_FILE);
@@ -1545,7 +1565,7 @@ void FFrame::OnPrintSetup(wxCommandEvent& WXUNUSED(event))
 
 void FFrame::OnPrint(wxCommandEvent& WXUNUSED(event))
 {
-    if (plot_pane->is_background_white())
+    if (!plot_pane->is_background_white())
         if (wxMessageBox ("Plots will be printed on white background, \n"
                             "to save ink/toner.\n"
                             "Now background of your plot on screen\n"
@@ -1582,9 +1602,9 @@ void FFrame::output_text(OutputStyle style, const string& str)
     io_pane->append_text(style, str.c_str());
 }
 
-void FFrame::refresh_plots(bool refresh, bool update)
+void FFrame::refresh_plots(bool refresh, bool update, bool only_main)
 {
-    plot_pane->refresh_plots(refresh, update);
+    plot_pane->refresh_plots(refresh, update, only_main);
 }
 
 void FFrame::draw_crosshair(int X, int Y)
@@ -1733,9 +1753,7 @@ FSetDlg::FSetDlg(wxWindow* parent, const wxWindowID id, const wxString& title,
     wxButton *btCancel = new wxButton (this, wxID_CANCEL, "Cancel");
     sizerH->Add (btCancel, 0, wxALL|wxALIGN_CENTER, 5 );
     sizer0->Add (sizerH, 0, wxALL|wxALIGN_CENTER, 5);
-    SetAutoLayout (true);
     SetSizer (sizer0);
-    sizer0->Fit (this);
     sizer0->SetSizeHints (this);
 }
 
@@ -1767,7 +1785,7 @@ FToolBar::FToolBar (wxFrame *parent, wxWindowID id)
     ToggleTool(ID_ft_m_range, m == mmd_range);
     AddRadioTool(ID_ft_m_bg, "Background", wxBitmap(baseline_t_xpm), 
                                                                  wxNullBitmap, 
-                 "Baseline Mode", "Use mouse for substracting background"); 
+                 "Baseline Mode", "Use mouse for subtracting background"); 
     ToggleTool(ID_ft_m_bg, m == mmd_bg);
     AddRadioTool(ID_ft_m_add, "Add peak", wxBitmap(addpeak_t_xpm), wxNullBitmap,
                  "Add-Peak Mode", "Use mouse for adding new peaks"); 
@@ -1808,13 +1826,10 @@ FToolBar::FToolBar (wxFrame *parent, wxWindowID id)
              "Load data from file");
     AddSeparator();
     //background
-    AddTool (ID_ft_b_with, "With Bg", wxBitmap(plusbg_xpm), wxNullBitmap,  
-             wxITEM_CHECK, "Plot with/without background",
-             "Draw plot with added (or not) baseline");
-    ToggleTool (ID_ft_b_with, my_core->plus_background);
-    AddTool (ID_D_B_CLEAR, "Clear Bg", wxBitmap(clear_xpm), wxNullBitmap, 
-             wxITEM_NORMAL, "Clear background", 
-             "Delete all baseline points");
+    AddTool (ID_ft_b_strip, "Strip Bg", wxBitmap(plusbg_xpm), wxNullBitmap,  
+             wxITEM_NORMAL, "Strip background",
+             "Remove selected background from data");
+    EnableTool(ID_ft_b_strip, (m == mmd_bg));
     AddSeparator();
     //sum
     AddTool (ID_S_INFO, "Tree", wxBitmap(tree_xpm), wxNullBitmap,
@@ -1850,8 +1865,6 @@ FToolBar::FToolBar (wxFrame *parent, wxWindowID id)
 
 void FToolBar::OnIdle(wxIdleEvent &event)
 {
-    if (GetToolState(ID_ft_b_with) != my_core->plus_background) 
-        ToggleTool(ID_ft_b_with, my_core->plus_background);
     event.Skip();
 }
 
@@ -1882,8 +1895,8 @@ void FToolBar::OnClickTool (wxCommandEvent& event)
         case ID_ft_v_pr:  
             frame->OnPreviousZoom(dummy_cmd_event);
             break;
-        case ID_ft_b_with: 
-            exec_command(S("o.plot ") + (event.IsChecked() ? "+" : "-")); 
+        case ID_ft_b_strip: 
+            frame->plot_pane->get_bg_manager()->strip_background();
             break; 
         case ID_ft_f_run : 
             exec_command("f.run"); 
@@ -1980,4 +1993,3 @@ void FStatusBar::set_hint(const char *left, const char *right)
 }
 
 
-
