@@ -1,5 +1,5 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
-// $Id: wx_gui.h,v 1.19 2005/03/04 14:28:47 wojdyr Exp $
+// $Id: wx_gui.h,v 1.29 2005/06/08 20:52:28 wojdyr Exp $
 
 #ifndef WX_GUI__H__
 #define WX_GUI__H__
@@ -110,14 +110,10 @@ public:
     void OnDXLoad        (wxCommandEvent& event);   
     void OnDRecent       (wxCommandEvent& event);
     void OnDEditor       (wxCommandEvent& event);
+    void OnFastDT        (wxCommandEvent& event);
+    void OnFastDTUpdate  (wxUpdateUIEvent& event);           
     void OnDInfo         (wxCommandEvent& event);
-    void OnDDeviation    (wxCommandEvent& event);
-    void OnDRange        (wxCommandEvent& event);
-    void OnDBInfo        (wxCommandEvent& event);
-    void OnDBClear       (wxCommandEvent& event);
-    void OnDCalibrate    (wxCommandEvent& event); 
     void OnDExport       (wxCommandEvent& event);
-    void OnDSet          (wxCommandEvent& event); 
 
     void OnSHistory      (wxCommandEvent& event);            
     void OnSInfo         (wxCommandEvent& event);         
@@ -158,6 +154,9 @@ public:
     void OnChangeMouseMode (wxCommandEvent& event);
     void OnModePeak      (wxUpdateUIEvent& event);
     void OnChangePeakType(wxCommandEvent& event);
+    void OnStripBg       (wxCommandEvent& event);
+    void OnClearBg       (wxCommandEvent& event);
+    void OnSplineBg      (wxCommandEvent& event);
     void OnGViewAll      (wxCommandEvent& event);
     void OnGFitHeight    (wxCommandEvent& event);
     void OnGScrollLeft   (wxCommandEvent& event);
@@ -189,11 +188,13 @@ public:
     void output_text(OutputStyle style, const std::string& str);
     void change_zoom(const std::string& s);
     void scroll_view_horizontally(fp step);
-    void refresh_plots(bool refresh=true, bool update=false);
+    void refresh_plots(bool refresh=true, bool update=false, 
+                       bool only_main=false);
     void draw_crosshair(int X, int Y);
     void focus_input();
     void set_status_text(const wxString &text, StatusBarField field=sbf_text) 
             { if (status_bar) SetStatusText(text, field); }
+    bool display_help_section(const std::string &s);
 
 protected:
     ProportionalSplitter *main_pane;
@@ -204,8 +205,6 @@ protected:
 
     int peak_type_nr;
     FToolBar *toolbar;
-    FDXLoadDlg *dxload_dialog;
-    DataEditorDlg *data_editor;
     ProportionalSplitter *v_splitter;
     wxPrintData *print_data;
     wxPageSetupData* page_setup_data;
@@ -216,7 +215,7 @@ protected:
 #endif
     std::string last_include_path;
     std::list<wxFileName> recent_data_files;
-    wxMenu *data_menu_recent;
+    wxMenu *data_menu_recent, *data_ft_menu;
 
     void place_plot_and_io_windows(wxWindow *parent);
     void create_io_panel(wxWindow *parent);
@@ -230,7 +229,6 @@ protected:
 DECLARE_EVENT_TABLE()
 };
 
-
 void add_peak(fp height, fp ctr, fp hwhm);
 void add_peak_in_range(fp xmin, fp xmax);
 extern FFrame *frame;
