@@ -1,4 +1,6 @@
 // This file is part of fityk program. Copyright (C) Marcin Wojdyr
+// $Id: wx_pane.cpp,v 1.25 2005/06/03 19:29:51 wojdyr Exp $
+
 
 // wxwindows headers, see wxwindows samples for description
 #include <wx/wxprec.h>
@@ -9,15 +11,15 @@
 #include <wx/wx.h>
 #endif
 
-#include "common.h"
-RCSID ("$Id: wx_pane.cpp,v 1.20 2005/03/04 14:28:47 wojdyr Exp $")
-
 #include <wx/colordlg.h>
 #include <wx/fontdlg.h>
 #include <wx/treectrl.h>
+
+#include "common.h"
 #include "wx_pane.h" 
 #include "wx_gui.h" 
 #include "wx_plot.h" 
+#include "wx_mplot.h" 
 #include "data.h" 
 #include "pcore.h" 
 #include "other.h" 
@@ -96,18 +98,26 @@ void PlotPane::save_settings(wxConfigBase *cf) const
 void PlotPane::read_settings(wxConfigBase *cf)
 {
     cf->SetPath("/PlotPane");
-    SetProportion(from_config_read_double(cf, "PlotPaneProportion", 0.75));
-    aux_split->SetProportion(from_config_read_double(cf, "AuxPlotsProportion", 
+    SetProportion(read_double_from_config(cf, "PlotPaneProportion", 0.75));
+    aux_split->SetProportion(read_double_from_config(cf, "AuxPlotsProportion", 
                                                          0.5));
-    show_aux(0, from_config_read_bool(cf, "ShowAuxPane0", true));
-    show_aux(1, from_config_read_bool(cf, "ShowAuxPane1", false));
+    show_aux(0, read_bool_from_config(cf, "ShowAuxPane0", true));
+    show_aux(1, read_bool_from_config(cf, "ShowAuxPane1", false));
     plot->read_settings(cf);
     for (int i = 0; i < 2; ++i)
         aux_plot[i]->read_settings(cf);
 }
 
-void PlotPane::refresh_plots(bool refresh, bool update)
+void PlotPane::refresh_plots(bool refresh, bool update, bool only_main)
 {
+    if (only_main) {
+        if (refresh)
+            plot->Refresh(false);
+        if (update) 
+            plot->Update();
+        return;
+    }
+    // not only main
     vector<FPlot*> vp = get_visible_plots();
     for (vector<FPlot*>::const_iterator i = vp.begin(); i != vp.end(); ++i) {
         if (refresh)
@@ -137,6 +147,8 @@ bool PlotPane::is_background_white()
     return true;
 }
 
+BgManager* PlotPane::get_bg_manager() { return plot; }
+
 const std::vector<FPlot*> PlotPane::get_visible_plots() const
 {
     vector<FPlot*> visible;
@@ -183,7 +195,8 @@ void PlotPane::show_aux(int n, bool show)
     }
 }
 
-// draw "crosshair cursor" -> erase old and draw new
+
+/// draw "crosshair cursor" -> erase old and draw new
 void PlotPane::draw_crosshair(int X, int Y)
 {
     static bool drawn = false;
@@ -229,10 +242,8 @@ IOPane::IOPane(wxWindow *parent, wxWindowID id)
                             wxWANTS_CHARS|wxTE_PROCESS_ENTER|wxTE_PROCESS_TAB);
     io_sizer->Add (input_field, 0, wxEXPAND);
 
-    SetAutoLayout (true);
-    SetSizer (io_sizer);
-    io_sizer->Fit (this);
-    io_sizer->SetSizeHints (this);
+    SetSizer(io_sizer);
+    io_sizer->SetSizeHints(this);
 }
 
 void IOPane::save_settings(wxConfigBase *cf) const
@@ -260,10 +271,8 @@ END_EVENT_TABLE()
     tree = new DataPaneTree(this, ID_DATAPANE_TREE);
     sizer->Add(tree, 1, wxEXPAND);
 
-    SetAutoLayout (true);
-    SetSizer (sizer);
-    sizer->Fit (this);
-    sizer->SetSizeHints (this);
+    SetSizer(sizer);
+    sizer->SetSizeHints(this);
 }
 //===============================================================
 //                            DataPaneTree
@@ -684,60 +693,6 @@ bool FPrintout::OnPrintPage(int page)
 }
 
 
-//===============================================================
-
-wxColour read_color_from_config(const wxConfigBase *config, const wxString& key,
-                                 const wxColour& default_value)
-{
-    return wxColour (config->Read (key + "/Red", default_value.Red()), 
-                     config->Read (key + "/Green", default_value.Green()), 
-                     config->Read (key + "/Blue", default_value.Blue()));
-}
-
-void write_color_to_config (wxConfigBase *config, const wxString& key,
-                            const wxColour& value)
-{
-    config->Write (key + "/Red", value.Red());
-    config->Write (key + "/Green", value.Green());
-    config->Write (key + "/Blue", value.Blue());
-}
-
-wxFont read_font_from_config (const wxConfigBase *config, const wxString& key,
-                              const wxFont& default_value)
-{
-    if (!default_value.Ok()) {
-        if (config->HasEntry(key+"/pointSize")
-              && config->HasEntry(key+"/family")
-              && config->HasEntry(key+"/style")
-              && config->HasEntry(key+"/weight")
-              && config->HasEntry(key+"/faceName"))
-            return wxFont (config->Read(key+"/pointSize", 0L),
-                           config->Read(key+"/family", 0L),
-                           config->Read(key+"/style", 0L),
-                           config->Read(key+"/weight", 0L),
-                           false, //underline
-                           config->Read(key+"/faceName", ""));
-        else
-            return wxNullFont;
-    }
-    return wxFont (config->Read(key+"/pointSize", default_value.GetPointSize()),
-                   config->Read(key+"/family", default_value.GetFamily()),
-                   config->Read(key+"/style", default_value.GetStyle()),
-                   config->Read(key+"/weight", default_value.GetWeight()),
-                   false, //underline
-                   config->Read(key+"/faceName", default_value.GetFaceName()));
-}
-
-void write_font_to_config (wxConfigBase *config, const wxString& key,
-                           const wxFont& value)
-{
-    config->Write (key + "/pointSize", value.GetPointSize());
-    config->Write (key + "/family", value.GetFamily());
-    config->Write (key + "/style", value.GetStyle());
-    config->Write (key + "/weight", value.GetWeight());
-    config->Write (key + "/faceName", value.GetFaceName());
-}
-
 //===============================================================
 //                            ProportionalSplitter
 //===============================================================
